//+------------------------------------------------------------------+
//|                           Infinity Hull Moving Average V1.03.mq5 |
//|                                  Copyright 2021, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.03"

#include <MovingAverages.mqh>

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_plots   1
//--- plot Label1
#property indicator_label1  "Hull Moving Average Color"
#property indicator_type1   DRAW_COLOR_LINE
#property indicator_color1  clrMediumBlue,clrGold
#property indicator_color1  Blue,Gold
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2


//--- input parameters
input int                  HullPeriod = 190;
input ENUM_MA_METHOD       HullMethod = MODE_SMMA;
input ENUM_APPLIED_PRICE   HullPrice  = PRICE_CLOSE;
input int                  HullShift  = 0;

//--- indicator buffers

double   hull[];
double   hullc[];
double   mahalf[];
double   mafull[];
double   hma[];

int   hull_handle1;
int   hull_handle2;
int   hull_handle3;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,hull,INDICATOR_DATA);
   SetIndexBuffer(1,hullc,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(2,mahalf,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,mafull,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,hma,INDICATOR_CALCULATIONS);

   PlotIndexGetInteger(0,PLOT_LINE_COLOR,1);

  
   string shortname;

   StringConcatenate(shortname,"HMA(",HullPeriod,")","(",HullMethod,")");
   
//--- set a label do display in DataWindow
   PlotIndexSetString(0,PLOT_LABEL,shortname);

//--- set a name to show in a separate sub-window or a pop-up help
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);

   
//--- set accuracy of displaying the indicator values
   IndicatorSetInteger(INDICATOR_DIGITS,2);
//--- bar, starting from which the indicator is drawn
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,HullPeriod-1);
 
   
   int halfperiod = HullPeriod/2;
   
   hull_handle1=iMA(Symbol(),0,HullPeriod,HullShift,HullMethod,HullPrice);
   hull_handle2=iMA(Symbol(),0,halfperiod,HullShift,HullMethod,HullPrice);
  
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
//--- if the size of price[] is too small
   if(rates_total<HullPeriod) return(0); // do not calculate or draw anything
//---   
   int sqrtperiod = (int)MathFloor(MathSqrt(HullPeriod));

//--- we can copy not all data

   int to_copy;
   if(prev_calculated>rates_total || prev_calculated<=0) to_copy=rates_total;
   else
     {
      to_copy=rates_total-prev_calculated;
      //--- last value is always copied
      to_copy++;
     }
//--- try to copy
   
   if(IsStopped()) // checking for stop flag
      return(0);
   if(CopyBuffer(hull_handle1,0,0,to_copy,mafull)<=0)
     {
      Print("Getting handle1 is failed! Error ",GetLastError());
      return(0);
     }

//---
   if(IsStopped()) // checking for stop flag
      return(0);
   if(CopyBuffer(hull_handle2,0,0,to_copy,mahalf)<=0)
     {
      Print("Getting handle2 is failed! Error ",GetLastError());
      return(0);
     }

//--- now calculate the indicator values
   int start;
   
   if(prev_calculated==0) start=HullPeriod-1; // set the starting index for input arrays
   else start=prev_calculated-1;    // set 'start' equal to the last index in the arrays
   for(int i=start;i<rates_total;i++)
     {
      hma[i]=(2*mahalf[i])-(mafull[i]);
      
     }
   if(HullMethod == MODE_SMA)
     {
    hull_handle3 = SimpleMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod,hma,hull);
     }
   if(HullMethod == MODE_EMA)
     {
      hull_handle3 = ExponentialMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod,hma,hull);
     }   
   if(HullMethod == MODE_SMMA)
     {
      hull_handle3 = SmoothedMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod,hma,hull);
     }
   if(HullMethod == MODE_LWMA)
     {
      hull_handle3 = LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,sqrtperiod,hma,hull);
     }                       
//--- return value of prev_calculated for next call

   //--- main loop of calculations
   for(int a=0; a<rates_total-1 && !IsStopped(); a++)
     {
      hullc[a]=hullc[a+1];
      if (hull[a] > hull[a+1]) hullc[a] = 1;
      if (hull[a] < hull[a+1]) hullc[a] = 0;
     }

   return(rates_total);
  }
//+------------------------------------------------------------------+
